@@ -6,18 +6,6 @@ import matplotlib.animation as animation
 from mpl_toolkits.mplot3d.art3d import Poly3DCollection
 from pykin.utils import transform_utils as tf
 
-try:
-    import fcl
-except ImportError:
-    warnings.warn(
-        "Cannot display mesh. Library 'fcl' not installed.")
-
-try:
-    import trimesh
-except ImportError:
-    warnings.warn(
-        "Cannot display mesh. Library 'trimesh' not installed.")
-
 # Colors of each directions axes. For ex X is green
 directions_colors = ["green", "cyan", "orange"]
 
@@ -78,71 +66,64 @@ def plot_basis(robot=None, ax=None):
     ax.plot([0, 0], [0, 0], [0, offset * 1.5],
             c=directions_colors[2], label="Z")
 
+def plot_robot(
+    robot, 
+    ax=None, 
+    transformations=None, 
+    visible_collision=False, 
+    visible_text=True,
+    visible_scatter=True):
 
-def plot_robot(robot, transformations=None, ax=None, name=None, visible_visual=False, visible_collision=False, mesh_path='../asset/urdf/baxter/'):
     """
     Plot robot
     """
-    
     if transformations is None:
-        transformations = robot.transformations
+        transformations = robot.init_transformations
 
+    name = robot.robot_name
     plot_basis(robot, ax)
+    
     links = []
     nodes = []
     transformation_matrix = []
 
     for i, (link, transformation) in enumerate(transformations.items()):
         links.append(link)
-        transformation_matrix.append(transformation.homogeneous_matrix)
+        transformation_matrix.append(transformation.h_mat)
 
-    for link, matrix in zip(links, transformation_matrix):
+    eef_idx = 0
+
+    for i, (link, matrix) in enumerate(zip(links, transformation_matrix)):
         nodes.append(tf.get_pos_mat_from_homogeneous(matrix))
+        if link == robot.eef_name:
+            eef_idx=i
 
     if name == "baxter":
-        plot_baxter(nodes, ax)
+        plot_baxter(nodes, ax, visible_text, visible_scatter)
     else:
         lines = ax.plot([x[0] for x in nodes], [x[1] for x in nodes], [
-            x[2] for x in nodes], linewidth=5, label=name)
+            x[2] for x in nodes], linewidth=2, label=name)
 
-        label = '(%0.4f, %0.4f, %0.4f)' % (
-            nodes[-1][0], nodes[-1][1], nodes[-1][2])
+        if visible_text:
+            label = '(%0.4f, %0.4f, %0.4f)' % (
+                nodes[eef_idx][0], nodes[eef_idx][1], nodes[eef_idx][2])
 
-        ax.text(nodes[-1][0], nodes[-1][1],
-                nodes[-1][2], label, size="8")
+            ax.text(nodes[eef_idx][0], nodes[eef_idx][1],
+                    nodes[eef_idx][2], label, size="8")
         
-        ax.scatter([x[0] for x in nodes], [x[1] for x in nodes],
-            [x[2] for x in nodes], s=55, c=lines[0].get_color())
+        if visible_scatter:
+            ax.scatter([x[0] for x in nodes], [x[1] for x in nodes],
+                [x[2] for x in nodes], s=20, c=lines[0].get_color())
     
-    if visible_visual:
-        plot_mesh(robot, transformations, mesh_path)
 
     if visible_collision:
         plot_collision(robot, transformations, ax)
 
-def plot_animation(robot, trajectory, interval=100, repeat=False, results=None):
-    """
-    Plot animation
-    """
-    fig = plt.figure(figsize = (12, 6), dpi = 100)
-    ax = fig.add_subplot(111, projection='3d')
-
-    def update(i):
-        print(f"{i/len(trajectory) * 100:.1f} %")
-        
-        if results is not None:
-            print(results[i])
-
-        if i == len(trajectory)-1:
-            print(f"{i/(len(trajectory)-1) * 100:.1f} %")
-            print("Animation Finished..")
-        ax.clear()
-        plot_robot(robot, trajectory[i], ax, name="baxter", visible_collision=True)
-    ani = animation.FuncAnimation(fig, update, np.arange(len(trajectory)), interval=interval, repeat=repeat)
-    plt.show()
+    ax.legend()
 
 
-def plot_baxter(nodes, ax):
+
+def plot_baxter(nodes, ax, visible_text=True, visible_scatter=True):
     """
     Plot baxter robot
     """
@@ -161,35 +142,115 @@ def plot_baxter(nodes, ax):
     left_lines = ax.plot([x[0] for x in left_nodes], [x[1] for x in left_nodes], [
         x[2] for x in left_nodes], linewidth=5, label="left arm")
 
-    head_label = '(%0.4f, %0.4f, %0.4f)' % (
-        head_nodes[-1][0], head_nodes[-1][1], head_nodes[-1][2])
-    pedestal_label = '(%0.4f, %0.4f, %0.4f)' % (
-        pedestal_nodes[-1][0], pedestal_nodes[-1][1], pedestal_nodes[-1][2])
-    right_label = '(%0.4f, %0.4f, %0.4f)' % (
-        right_nodes[8][0], right_nodes[8][1], right_nodes[8][2])
-    left_label = '(%0.4f, %0.4f, %0.4f)' % (
-        left_nodes[8][0], left_nodes[8][1], left_nodes[8][2])
+    if visible_text:
+        head_label = '(%0.4f, %0.4f, %0.4f)' % (
+            head_nodes[-1][0], head_nodes[-1][1], head_nodes[-1][2])
+        pedestal_label = '(%0.4f, %0.4f, %0.4f)' % (
+            pedestal_nodes[-1][0], pedestal_nodes[-1][1], pedestal_nodes[-1][2])
+        right_label = '(%0.4f, %0.4f, %0.4f)' % (
+            right_nodes[8][0], right_nodes[8][1], right_nodes[8][2])
+        left_label = '(%0.4f, %0.4f, %0.4f)' % (
+            left_nodes[8][0], left_nodes[8][1], left_nodes[8][2])
 
-    ax.text(head_nodes[-1][0], head_nodes[-1][1],
-            head_nodes[-1][2], head_label, size="8")
-    ax.text(pedestal_nodes[-1][0], pedestal_nodes[-1][1],
-        pedestal_nodes[-1][2], pedestal_label, size="8")
-    ax.text(right_nodes[-1][0], right_nodes[-1][1],
-            right_nodes[-1][2], right_label, size="8")
-    ax.text(left_nodes[-1][0], left_nodes[-1][1],
-            left_nodes[-1][2], left_label, size="8")
+        ax.text(head_nodes[-1][0], head_nodes[-1][1],
+                head_nodes[-1][2], head_label, size="8")
+        ax.text(pedestal_nodes[-1][0], pedestal_nodes[-1][1],
+            pedestal_nodes[-1][2], pedestal_label, size="8")
+        ax.text(right_nodes[-1][0], right_nodes[-1][1],
+                right_nodes[-1][2], right_label, size="8")
+        ax.text(left_nodes[-1][0], left_nodes[-1][1],
+                left_nodes[-1][2], left_label, size="8")
 
-    ax.scatter([x[0] for x in head_nodes], [x[1] for x in head_nodes], 
-        [x[2] for x in head_nodes], s=55, c=head_lines[0].get_color())
-    ax.scatter([x[0] for x in pedestal_nodes], [x[1] for x in pedestal_nodes], 
-        [x[2] for x in pedestal_nodes], s=55, c=pedestal_lines[0].get_color())
-    ax.scatter([x[0] for x in right_nodes], [x[1] for x in right_nodes], 
-        [x[2] for x in right_nodes], s=55, c=right_lines[0].get_color())
-    ax.scatter([x[0] for x in left_nodes], [x[1] for x in left_nodes], 
-        [x[2] for x in left_nodes], s=55, c=left_lines[0].get_color())
+    if visible_scatter:
+        ax.scatter([x[0] for x in head_nodes], [x[1] for x in head_nodes], 
+            [x[2] for x in head_nodes], s=30, c=head_lines[0].get_color())
+        ax.scatter([x[0] for x in pedestal_nodes], [x[1] for x in pedestal_nodes], 
+            [x[2] for x in pedestal_nodes], s=30, c=pedestal_lines[0].get_color())
+        ax.scatter([x[0] for x in right_nodes], [x[1] for x in right_nodes], 
+            [x[2] for x in right_nodes], s=30, c=right_lines[0].get_color())
+        ax.scatter([x[0] for x in left_nodes], [x[1] for x in left_nodes], 
+            [x[2] for x in left_nodes], s=30, c=left_lines[0].get_color())
 
+def plot_trajectories(ax, path):
+    """
+    Plot plot_trajectories
+    """
+    ax.scatter([x for (x, y, z) in path], [y for (x, y, z) in path], [z for (x, y, z) in path], s=10, c='r')
 
-def plot_collision(robot, transformations, ax, alpha=0.5):
+def plot_animation(
+    robot, 
+    trajectory,
+    fig=None,
+    ax=None,
+    eef_poses=None,
+    obstacles=None,
+    visible_obstacles=False,
+    visible_collision=False,
+    visible_text=True,
+    visible_scatter=True,
+    interval=100, 
+    repeat=False,
+    result=None):
+
+    """
+    Plot animation
+    """
+
+    def update(i):
+        # print(f"{i/len(trajectory) * 100:.1f} %")
+    
+        if i == len(trajectory)-1:
+            # print(f"{i/(len(trajectory)-1) * 100:.1f} %")
+            print("Animation Finished..")
+        ax.clear()
+
+        if visible_obstacles and obstacles:
+            plot_obstacles(obstacles, ax)
+        
+        if eef_poses is not None:
+            plot_trajectories(ax, eef_poses)
+          
+        if result is not None:
+            print(result[i])
+            print()
+
+        plot_robot(
+            robot, 
+            transformations=trajectory[i], 
+            ax=ax, 
+            visible_collision=visible_collision,
+            visible_text=visible_text,
+            visible_scatter=visible_scatter)
+
+    ani = animation.FuncAnimation(fig, update, np.arange(len(trajectory)), interval=interval, repeat=repeat)
+    plt.show()
+
+def plot_obstacles(obstacles, ax):    
+    """
+    Plot obstacles
+    """
+    import trimesh
+    from pykin.kinematics.transform import Transform
+    for key, value in obstacles:
+        o_type = value[0]
+        o_param = value[1]
+        o_pose = value[2]
+
+        if not isinstance(o_pose, Transform):
+            raise TypeError("Check obstacle pose type..")
+
+        if o_type == "mesh":
+            plot_mesh(ax, mesh=o_param, A2B=o_pose.h_mat)
+        if o_type == "sphere":
+            plot_sphere(ax, radius=o_param, p=o_pose, alpha=0.8, color='g')
+        if o_type == "box":
+            A2B = tf.get_h_mat(o_pose)
+            plot_box(ax, size=o_param, A2B=A2B, alpha=0.8, color='b')
+        if o_type == "cylinder":
+            A2B = tf.get_h_mat(o_pose.pos)
+            plot_cylinder(ax, radius=o_param[0], length=o_param[1], A2B=A2B, n_steps=100, alpha=0.8, color='r')
+
+def plot_collision(robot, transformations, ax, alpha=0.8):
     """
     Plot robot's collision
     """
@@ -202,7 +263,7 @@ def plot_collision(robot, transformations, ax, alpha=0.5):
         return color
 
     for link, transformation in transformations.items():
-        A2B = np.dot(transformation.homogeneous_matrix, robot.links[link].collision.offset.homogeneous_matrix)
+        A2B = np.dot(transformation.h_mat, robot.links[link].collision.offset.h_mat)
         color = _get_color(robot.links[link].visual.gparam)
 
         if robot.links[link].collision.gtype == 'cylinder':
@@ -270,6 +331,9 @@ def plot_box(ax=None, size=np.ones(3), alpha=1.0, A2B=np.eye(4), color="k"):
     Plot box
     """
     color = _check_color_type(color)
+    
+    if not isinstance(size, np.ndarray):
+        size = np.array(size)
 
     corners = np.array([
         [0, 0, 0],
@@ -310,43 +374,53 @@ def plot_box(ax=None, size=np.ones(3), alpha=1.0, A2B=np.eye(4), color="k"):
     p3c.set_facecolor(color)
     ax.add_collection3d(p3c)
 
-
-def plot_mesh(robot, transformations, mesh_path):
+def plot_rrt_vertices(vertices, ax):
     """
-    Plot mesh
+    Plot rrt* trees
     """
-    scene = trimesh.Scene()
-    for link, transformation in transformations.items():
-        if robot.links[link].visual.gtype == "mesh":
-            mesh_name = robot.links[link].visual.gparam.get('filename')
-            filename = mesh_path + mesh_name
-            A2B = np.dot(transformation.homogeneous_matrix, robot.links[link].visual.offset.homogeneous_matrix)
-            visual_color = robot.links[link].visual.gparam.get('color')
-            color = np.array([0.2, 0.2, 0.2, 1.])
-            if visual_color is not None:
-                color = np.array([color for color in visual_color.values()]).flatten()
-            scene = convert_trimesh_scene(scene, filename, A2B, color)
-            scene.set_camera(np.array([np.pi/2, 0, np.pi/2]), 5)
-    scene.show()
+    for vertex in vertices:
+        ax.plot([x for (x, y, z) in vertex],[y for (x, y, z) in vertex], [z for (x, y, z) in vertex],'k', linewidth=0.1,)
 
-
-def convert_trimesh_scene(scene, filename=None, A2B=np.eye(4), color="k"):
+def plot_path_planner(path, ax):
     """
-    Convert scene from mesh to trimesh
+    Plot rrt* path planner
     """
-    mesh = trimesh.load(filename)
-    color = _check_color_type(color)
-    mesh.visual.face_colors = color
-    scene.add_geometry(mesh, transform=A2B)
+    if path is None:
+        print("cannot create path")
+        return
 
-    return scene
+    ax.scatter([x for (x, y, z) in path], [y for (x, y, z) in path], [z for (x, y, z) in path], s=10, c='r')
+    ax.plot([x for (x, y, z) in path], [y for (x, y, z) in path], [z for (x, y, z) in path], '-b', linewidth=0.5,)
+    ax.text(path[0][0], path[0][1], path[0][2], 'Start', verticalalignment='bottom', horizontalalignment='center', size="20")
+    ax.text(path[-1][0], path[-1][1], path[-1][2],'Goal', verticalalignment='bottom', horizontalalignment='center', size="20")
 
 
-def init_3d_figure(name=None):
+def plot_mesh(ax=None, mesh=None, A2B=np.eye(4),
+              s=np.array([1.0, 1.0, 1.0]), ax_s=1, wireframe=False,
+              convex_hull=False, alpha=1.0, color="k"):
+
+    try:
+        import trimesh
+    except ImportError:
+        warnings.warn(
+            "Cannot display mesh. Library 'trimesh' not installed.")
+        return ax
+
+    vertices = mesh.vertices * s
+    vertices = np.hstack((vertices, np.ones((len(vertices), 1))))
+    vertices = np.dot(vertices, A2B.T)[:, :3]
+    vectors = np.array([vertices[[i, j, k]] for i, j, k in mesh.faces])
+
+    surface = Poly3DCollection(vectors)
+    surface.set_facecolor(color)
+    surface.set_alpha(alpha)
+    ax.add_collection3d(surface)
+
+def init_3d_figure(name=None, figsize=(15,7.5), dpi= 80):
     """
     Initializes 3d figure
     """
-    fig = plt.figure(name)
+    fig = plt.figure(name, figsize=figsize, dpi= dpi)
     ax = fig.add_subplot(111, projection='3d')
     return fig, ax
 
